<?php

namespace Ana\Boxes\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Ana\Boxes\Http\Requests\BoxRequest;
use Ana\Boxes\Interfaces\BoxRepositoryInterface;
use Ana\Boxes\Models\Box;

/**
 * Resource controller class for box.
 */
class BoxResourceController extends BaseController
{

    /**
     * Initialize box resource controller.
     *
     * @param type BoxRepositoryInterface $box
     *
     * @return null
     */
    public function __construct(BoxRepositoryInterface $box)
    {
        parent::__construct();
        $this->repository = $box;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Ana\Boxes\Repositories\Criteria\BoxResourceCriteria::class);
    }

    /**
     * Display a list of box.
     *
     * @return Response
     */
    public function index(BoxRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Ana\Boxes\Repositories\Presenter\BoxPresenter::class)
                ->$function();
        }

        $boxes = $this->repository->paginate();

        return $this->response->title(trans('boxes::box.names'))
            ->view('boxes::box.index', true)
            ->data(compact('boxes', 'view'))
            ->output();
    }

    /**
     * Display box.
     *
     * @param Request $request
     * @param Model   $box
     *
     * @return Response
     */
    public function show(BoxRequest $request, Box $box)
    {

        if ($box->exists) {
            $view = 'boxes::box.show';
        } else {
            $view = 'boxes::box.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('boxes::box.name'))
            ->data(compact('box'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new box.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BoxRequest $request)
    {

        $box = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('boxes::box.name')) 
            ->view('boxes::box.create', true) 
            ->data(compact('box'))
            ->output();
    }

    /**
     * Create new box.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BoxRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $box                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('boxes::box.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('boxes/box/' . $box->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/boxes/box'))
                ->redirect();
        }

    }

    /**
     * Show box for editing.
     *
     * @param Request $request
     * @param Model   $box
     *
     * @return Response
     */
    public function edit(BoxRequest $request, Box $box)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('boxes::box.name'))
            ->view('boxes::box.edit', true)
            ->data(compact('box'))
            ->output();
    }

    /**
     * Update the box.
     *
     * @param Request $request
     * @param Model   $box
     *
     * @return Response
     */
    public function update(BoxRequest $request, Box $box)
    {
        try {
            $attributes = $request->all();

            $box->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('boxes::box.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('boxes/box/' . $box->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('boxes/box/' . $box->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the box.
     *
     * @param Model   $box
     *
     * @return Response
     */
    public function destroy(BoxRequest $request, Box $box)
    {
        try {

            $box->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('boxes::box.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('boxes/box/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('boxes/box/' . $box->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple box.
     *
     * @param Model   $box
     *
     * @return Response
     */
    public function delete(BoxRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('boxes::box.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('boxes/box'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/boxes/box'))
                ->redirect();
        }

    }

    /**
     * Restore deleted boxes.
     *
     * @param Model   $box
     *
     * @return Response
     */
    public function restore(BoxRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('boxes::box.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/boxes/box'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/boxes/box/'))
                ->redirect();
        }

    }

}
